/' (c) 2022 cepstrum.co.jp           '/
/' This is not free software.        '/
'
' impulse response measurement program, 16bit, 8kHz, monoral
'
' GUI program, command line option : fbc -s gui
'
'
'                                               d
'                  +--->> D/A out     A/D in >>---+
'                  |                              | 
'                  |                              |
'                x |                           +  v  e 
'  random noise ---+                             (+)---+
'                  |               ^           -  ^    | 
'                  |               |              |    |
'                  |            +-----+ y         |    |
'                  |       +--->| FIR +-----------+    |
'                  |       |    +-----+                |
'                  +-------+       |                   |
'                          |    +-----+                |
'                          +--->| LMS |<---------------+
'                               +-----+
'                           
'
'--------------------------------------------------------------------------

#inclib "c_mix_impres2"

extern "C"
  declare sub c_routine(c_ad as single, byref c_da as single, byref xbuf as single, byref w as single, init as uinteger<32>)
end extern

'--------------------------------------------------------------------------

#include "portaudio.bi"
#include "graphics.bi"

#define FS          8000
#define PA_BUFLEN   1

#define LMSLEN 4000

type User_Data_Type
  dummy as any ptr
end type

dim user_data as User_Data_Type

dim stream    as any ptr 
dim errcode   as integer<32>

dim inchar    as string
dim keycode   as integer
dim i         as integer
dim plotx     as integer
dim ploty     as integer
dim yscale    as single = 1.0

dim maxval    as single 
dim maxpos    as integer 
dim x         as integer
dim y         as integer
dim buttons   as integer

dim shared xbuf(LMSLEN*2) as single
dim shared w(LMSLEN)      as single

dim shared    as single  wcopy(LMSLEN)
dim shared    as integer wcopy_busy = 0
dim shared    as integer frame_count = 0

dim tmp as single

'==========================================================================

function callbackfunc cdecl(                    _
  pread       as integer<16> ptr,               _
  pwrite      as integer<16> ptr,               _
  rwcount     as uinteger<32>,                  _
  timeInfo    as PaStreamCallbackTimeInfo ptr,  _
  flag        as uinteger<32>,                  _
  puser_data  as User_Data_Type ptr             _
) as integer

  static power          as single      = 100.0
  static fir_ix         as integer<64> = 0
  static init_flag      as integer     = 1
  dim    i_ad           as integer<16>
  dim    i_da           as integer<16>
  dim    f_ad           as single
  dim    f_da           as single
  dim    x              as single
  dim    y              as single
  dim    d              as single
  dim    e              as single  
  dim    i              as integer

  i_ad=*pread
  f_ad=i_ad/32768.0
  
  c_routine(f_ad, f_da, xbuf(0), w(0), 0)

  i_da=int(f_da*32768.0)   ' int() : floor function
  *pwrite=i_da

  if wcopy_busy=0 then
    for i=0 to LMSLEN-1
      wcopy(i)=w(i)
    next
  end if

  frame_count=frame_count+1
   
  return 0

end function

'==========================================================================

windowtitle " [cepstrum.co.jp]"
screenres 500+1, 400+1

errcode=Pa_Initialize()

errcode=Pa_OpenDefaultStream(  _ 
  @stream,                     _  ' PaStream ptr ptr        
  1,                           _  ' input channels          
  1,                           _  ' output channels       
  8,                           _  ' sample format (paInt16)  
  FS,                          _  ' Fs                   
  PA_BUFLEN,                   _  ' frames per buffer     
  @callbackfunc,               _  ' callback function  
  @user_data                   _  ' user data      
) 

c_routine(0, tmp, xbuf(0), w(0), 1)    ' initialize c_routine()

wcopy_busy=0
errcode=Pa_StartStream(stream)
do

  getmouse(x, y, , buttons)
  if buttons=1 then            ' mouse left button click
    yscale=0.7*yscale
  end if
  if buttons=2 then            ' mouse right button click
    yscale=1.4*yscale
  end if
  if yscale<0.1 then
    yscale=0.1
  end if
  if 10.0<yscale then
    yscale=10.0
  end if
  
  g_fillbox(0, 0, 500+1, 400+1, BLACK_COLOR)

  wcopy_busy=1
  maxval=0.0
  maxpos=0.0
  for i=0 to LMSLEN-1 
    if maxval<abs(wcopy(i)) then
      maxval=abs(wcopy(i))
      maxpos=i
    end if
    plotx=i/8
    ploty=yscale*wcopy(i)*100+200
    g_line(plotx, 200, plotx, ploty, GREEN_HIGH_COLOR)
  next
  g_string(4, 388, "peak pos : "+str(maxpos), YELLOW_COLOR)
  g_string(4, 16, "click mouse button to change Y-axis scale", YELLOW_COLOR)
  g_string(4, 4, "hit ctrl-C or ESC key to exit", YELLOW_COLOR)
  wcopy_busy=0

  inchar=inkey$
  keycode=asc(inchar)
  if keycode=3 or keycode=27 then           ' if ctrl-C or ESC
    errcode=Pa_StopStream(stream)
    errcode=Pa_CloseStream(stream)
    Pa_Terminate()
    '-------------------------------
    open "out.txt" for output as #1
    for i=0 to LMSLEN-1
      print #1, i, wcopy(i)
    next
    close #1
    '-------------------------------
    system
  end if

  sleep(200)

loop


